"""
simple program that plays an interval and then says what it is

requires python (http://www.python.org/download/) and python-midi (http://www.mxm.dk/products/public/pythonmidi/download)

simplified running instructions:
1. download and install python
2. download and extract python-midi (7zip, from www.7zip.com, can open tar.gz files)
3. place this file in the "midi" directory
4. run this file with python

"""


import tempfile
import os
import random
import time

import MidiOutFile

random.seed()


interval_names = ['P1', 'm2', 'M2', 'm3', 'M3', 'P4', 'TT', 'P5', 'm6', 'M6', 'm7', 'M7', 'P8']


dissonances = ['no beats', 'very dissonant', 'dissonant', 'consonant', 'consonant', 'no beats', 'very dissonant']
def interval_dissonance(steps):
    if steps > 6:
        steps = 12 - steps
    return dissonances[steps]


def interval_size(steps):
    if steps == 6:
        return "medium"
    elif steps < 6:
        return "small"
    else:
        return "large"


#create a midi file for playing a given interval
def generate_midi(note1, note2):
    #copy/pasting example_minimal_type0 ftw
    
    filehandle, out_file = tempfile.mkstemp(".mid")
    os.close(filehandle) #mkstemp opens the file, but we don't need it open yet
    midi = MidiOutFile.MidiOutFile(out_file)

    midi.header()
    midi.start_of_track() 


    note_duration = 192
    note_gap = 64


    #play notes simultaneously
    midi.update_time(0)
    midi.note_on(channel=0, note=note1)
    midi.note_on(channel=0, note=note2)

    midi.update_time(note_duration)
    midi.note_off(channel=0, note=note1)
    midi.note_off(channel=0, note=note2)

    #ascending
    midi.update_time(note_gap)
    midi.note_on(channel=0, note=note1)
    
    midi.update_time(note_duration/2)
    midi.note_on(channel=0, note=note2)
    
    midi.update_time(note_duration)
    midi.note_off(channel=0, note=note1)
    midi.note_off(channel=0, note=note2)

    #descending
    midi.update_time(note_gap)
    midi.note_on(channel=0, note=note2)
    
    midi.update_time(note_duration/2)
    midi.note_on(channel=0, note=note1)
    
    midi.update_time(note_duration)
    midi.note_off(channel=0, note=note1)
    midi.note_off(channel=0, note=note2)


    midi.update_time(0)
    midi.end_of_track()

    midi.eof()
    
    return out_file


if os.name == 'nt':
    import ctypes
    
    mciExecute = ctypes.windll.winmm.mciExecute
    mciExecute.argTypes = [ctypes.c_char_p]
    
    def play_midi(filename):
        assert mciExecute("open sequencer!%s alias midifile wait" % filename)
        assert mciExecute("play midifile wait")
        assert mciExecute("close midifile wait")
    
    
else:
    def play_midi(filename):
        os.system("timidity '%s' &>/dev/null" % filename)


while True:
    print ""
    
    steps = random.randint(0,12)

    note1 = random.randint(0x30,0x50)
    note2 = note1 + steps

    midifile = generate_midi(note1, note2)

    play_midi(midifile)
    
    response = raw_input('enter a guess or "r" to replay: ')
    
    while response == 'r':
        play_midi(midifile)
        
        response = raw_input('enter a guess or "r" to replay: ')
    
    print "interval: %s (%s, %s)" % (interval_names[steps], interval_dissonance(steps), interval_size(steps))
    
    raw_input("press enter to continue.")
    
    os.unlink(midifile)

